using AceField.Scripts.Effects;
using AceField.Scripts.Logic;
using Godot;

namespace AceField.Scripts;

public partial class Player : CharacterBody2D
{
	private int _currentPlayerId = 1;

	[Export] public float MaxSpeed = 400f;
	[Export] public float Acceleration = 500f;
	[Export] public float Deceleration = 500f;
	[Export] public float RotationSpeed = 5f;

	[Export] public int Reach = 5;

	[Export] public double Health = 100;
	[Export] public double MaxHealth = 100;
	[Export] public double ActionPoints = 20;
	[Export] public double MaxActionPoints = 20;
	[Export] public double AmmoAmount = 30;
	[Export] public double MaxAmmoAmount = 30;

	[Export] public Camera2D PlayerCamera;
	[Export] public PlayerInput PlayerInput;

	[Export] public float PlayerDashAcceleration = 2f;

	[Export] public PackedScene BulletScene;

	[Export] public string PlayerName;

	[Export]
	public int PlayerId
	{
		get => _currentPlayerId;
		set
		{
			_currentPlayerId = value;
			PlayerInput.SetMultiplayerAuthority(value);
		}
	}

	public bool IsCurrentPlayer => _currentPlayerId == Multiplayer.GetUniqueId();

	public override void _Ready()
	{
		Health = MaxHealth;
		ActionPoints = MaxActionPoints;

		if (PlayerId == Multiplayer.GetUniqueId())
			PlayerCamera.Enabled = true;

		PlayerName ??= $"Player#{PlayerId}"; 
		GetNode<Label>("Overlay/NameTag").Text = PlayerName;

		GetNode<ProgressBar>("Overlay/HealthBar").Value = Health / MaxHealth * 100;
	}

	public override void _Process(double delta)
	{
		if (PlayerInput.IsShooting)
		{
			Rpc(nameof(Shoot));
			PlayerInput.IsShooting = false;
		}
	}

	public override void _PhysicsProcess(double delta)
	{
		var input = PlayerInput.MovementDirection;

		if (input != Vector2.Zero)
		{
			input = input.Normalized();
			Velocity = Velocity.MoveToward(input * MaxSpeed, Acceleration * (float)delta);

			var centre = GetNode<Node2D>("RotationCentre");
			var finalRotation = input.Angle() + Mathf.Pi / 2;
			centre.Rotation = Mathf.LerpAngle(centre.Rotation, finalRotation, RotationSpeed * (float)delta);
		}
		else
		{
			Velocity = Velocity.MoveToward(Vector2.Zero, Deceleration * (float)delta);
		}

		var dashCountdown = GetNode<Timer>("DashCountdown");
		if (PlayerInput.IsDashing && dashCountdown.IsStopped())
		{
			PlayerInput.IsDashing = false;
			Velocity *= PlayerDashAcceleration;
			dashCountdown.Start();
		}

		Position += Velocity * (float)delta;
		MoveAndSlide();
	}

	public void TakeDamage(double damage)
	{
		Rpc(nameof(GotDamage), damage);
	}

	[Rpc(MultiplayerApi.RpcMode.AnyPeer, CallLocal = true)]
	private void GotDamage(double damage)
	{
		var protection = GetNode<Timer>("ProtectionCountdown");

		if (protection.IsStopped())
		{
			Health -= damage;

			var tween = CreateTween();
			var bar = GetNode<ProgressBar>("Overlay/HealthBar");
			tween.TweenProperty(bar, "value", Health / MaxHealth * 100, 0.3);

			protection.Start();
		}

		var shakableCamera = GetNode<CameraShake>("Camera2D");
		shakableCamera.AddTrauma(0.5f);
	}

	[Rpc(MultiplayerApi.RpcMode.AnyPeer, CallLocal = true)]
	private void Shoot()
	{
		var marker = GetNode<Marker2D>("RotationCentre/Muzzle");
		var projectile = BulletScene.Instantiate<Bullet>();
		projectile.Transform = marker.GlobalTransform;
		projectile.PlayerId = PlayerId;

		GetParent().AddChild(projectile);
	}
}
